
############################################################
#                                                          #
#                          Libraries                       #
#                                                          #
############################################################
# 23-05-2023 PM2,5 de 2022
# Substitue all NA's in df_obs_sub by gbm predictions based on ok_total.

# Needs packages:
library(gbm)
library(lubridate)

############################################################
#                                                          #
#                        Initiall data                     #
#                                                          #
############################################################
pollutant       <- "pm2p5" #treure
years           <- c(2020, 2021, 2022) #revisar/afegir al config
save_result     <-"/esarchive/scratch/mhajji/Meriem/Gene/results_regional_" #afegir al config
path_gbm        <- "/esarchive/scratch/mhajji/Meriem/Gene/" #afegir al config
############################################################
#                                                          #
#                 Load Local Stations                      #
#                                                          #
############################################################
#We will pick the latest data available
#local_stations     <- readRDS(paste0(save_result,years[length(years)],"/results_gene_",pollutant, "/","local_stations_",file_format, "_",pollutant,".rds"))

############################################################
#                                                          #
#                  Load All observations                   #
#                                                          #
############################################################
#canviar start date i end date i dates
df_obs_years <- NULL
for(read_year in years){
  df_obs_gbm        <- readRDS(file = paste0(save_result,read_year,"/results_gene_",pollutant, "/","df_obs_",file_format,"_",pollutant,".rds"))
  
  start_date  <- as.Date(paste(read_year, "-01-01", sep = ""))
  end_date    <- as.Date(paste(read_year, "-12-30", sep = ""))
  
  dates      <- seq(start_date, end_date, by = "day")
  df_obs_sub_gbm <- subset(df_obs_gbm, date>=start_date & date <= end_date)
  
  
  # Comprovation we don't have duplications in obs
  if(any(duplicated(df_obs_sub_gbm$date))){
    print(paste0("Warning, there is duplicated observations on date :", df_obs_sub_gbm$date[duplicated(df_obs_sub_gbm$date)]))
    df_obs_sub_gbm <- distinct(df_obs_sub_gbm, date, .keep_all = TRUE)
    print("The df_obs_sub_gbm (alias=observations) have been modified to have the same length as the model")
  }
  
  if (length(df_obs_sub_gbm$date) != length(dates)){
    print("WARNING: There are missing hours/dates in observations. And the code is not developed for daily data yet!!")
  }
  
  df_obs_years <- rbind.fill(df_obs_years, df_obs_sub_gbm)
}

names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_years)))

############################################################
#                                                          #
#          Setting test and training data                  #
#                                                          #
############################################################
# # Create an empty data frame to store non-NA data
# df_not_na <- data.frame(date = as.Date(character(0)), stringsAsFactors = FALSE)
# 
# # Create an empty data frame to store NA data
# df_na     <- data.frame(date = as.Date(character(0)), stringsAsFactors = FALSE)
# 
# for (i in 1:(length(df_obs_years)-1)) {
#   station <- names(df_obs_years)[i]
#   station_data <- df_obs_years[, c("date", station)]
#   
#   # Filtrar los datos no nulos y añadirlos al data frame correspondiente
#   not_na_data <- station_data[!is.na(station_data[, station]), ]
#   df_not_na   <- rbind(df_not_na, not_na_data)
#   
#   # Filtrar los datos nulos y añadirlos al data frame correspondiente
#   na_data <- station_data[is.na(station_data[, station]), ]
#   df_na <- rbind(df_na, na_data)
# }
# 
# 
# 
# 
# 
# # Iterate over each station and separate non-NA and NA data
# for (s in 1:length(names_local_stations)) {
#   stations     <- names_local_stations[[s]]
#   index        <- which(stations==colnames(df_obs_years))
#   station_data <- df_obs_years[,index]
#   
#   # Separate non-NA data and add it to the corresponding data frame
#   non_na_data <- station_data[complete.cases(station_data), ]
#   df_not_na   <- merge(df_not_na, non_na_data, by = "date", all = TRUE)
#   
#   # Separate NA data and add it to the corresponding data frame
#   na_data <- station_data[is.na(station_data[, station]), ]
#   df_na <- merge(df_na, na_data, by = "date", all = TRUE)
# }
# 
# # Print the resulting data frames
# str(df_not_na)
# str(df_na)


############################################################
#                                                          #
#            Load raw model interpolated to AQMS           #
#                                                          #
############################################################
df_all_model_raw_at_AQMS_gbm <- NULL
for(read_year in years){
  
  read_model_raw_at_AQMS_df     <- readRDS(paste0(save_result,read_year,"/results_gene_",pollutant, "/","model_raw_at_AQMS_",file_format, "_",pollutant,"_df.rds"))
  df_all_model_raw_at_AQMS_gbm <- rbind.fill(df_all_model_raw_at_AQMS_gbm, read_model_raw_at_AQMS_df)
}

############################################################
#                                                          #
#   AQMS interpolated to AQMS as covariate for GBM         #
#                                                          #
############################################################

# We pick up the observations configured in config file:
obs_filled       <- df_obs_years 
AQMS_obs         <- subset(df_obs_years, select = -date) #we remove dates from observations data frame
OK_other_obs_df  <- NULL
list_gbm_results <- NULL

# To fill the gaps of observations in each station, so we need a loop to repeat the fill process in each station:
for (s in 1:length(names_local_stations)){
  station                 <- names_local_stations[[s]]
  
 # Arregment of AQMS interpolated to AQMS
 # station is the code station that we hidded to interpolate to it, first
  col_station           <- which(station==colnames(df_obs_years)) # index de l'estació en les obs i treure-la
  stations_to_interp    <- df_obs_years[,-col_station] #obs de les altres estacions sense l'estació a interpolar
  c <- 0
  #Loop over all days to interpolate date from other station to station
  for(j in 1:length(df_obs_years$date)){
    #print(paste0("Date: ", stations_to_interp$date[j]))
    #We take the data day by day /hour by hour
    ok_obs    <- stations_to_interp[j,] #obs per dia de totes les estacions menys la eliminada
    ok_obs    <- subset(ok_obs,select=-date)
    df_ok_obs <- data.frame(ref_obs = colnames(ok_obs), pollutant = as.numeric(ok_obs)) #data frame estacions x 1 dia
    
    #coordinates where to interpolate:
    df_hidded_AQMS                      <- local_stations[ which(local_stations$ref_obs == station) , ] #spatial PDF de l'estació a interpolar
  
    if (compute_in_UTM) {
      df_hidded_AQMS <- spTransform(df_hidded_AQMS, sp::CRS(crs_utm))
    }
    
    #    1.2 Compute Variogram
    #with other AQMS we want a SPDF that has the ref of the station and the coordinates of each, with the value of the pollution in that day/hour
    sp_other_ok_obs <- local_stations[ -which(local_stations$ref_obs == station) , ] #spatial PDF on hi han totes les coords excepte l'amagada (sobren estacions)
    df_other_obs <- as.data.frame(sp_other_ok_obs) #pasem de spdf a df per poder filtrar
    df_obs_data  <- merge(df_other_obs[, -which(names(df_other_obs) == "pollutant")], df_ok_obs, by = "ref_obs") #copiem les coordenades de les estacions d'interès amb les observacion que pertoquen
    
    if(!sum(is.na(df_obs_data$pollutant))>=length(df_obs_data$pollutant)){
      df_obs_data                   <- na.omit(df_obs_data)
      coordinates(df_obs_data)      <- ~lon+lat
      sp::proj4string(df_obs_data)  <- crs_latlon
      
      if (compute_in_UTM) {
        df_obs_data <- spTransform(df_obs_data, sp::CRS(crs_utm))
      }
      
      pollutant.fit_ok          <- automap::autofitVariogram(formula=pollutant~1, input_data=df_obs_data,model="Ste")
      
      ok_gstat <- gstat(formula=pollutant~1, data = df_obs_data, model = pollutant.fit_ok$var_model, nmax=100)
      OK       <- predict(object=ok_gstat, newdata=df_hidded_AQMS)
      
      
      OK_other_obs           <- data.frame(data=OK$var1.pred)
      colnames(OK_other_obs) <- station
      OK_other_obs$date      <- stations_to_interp$date[j]
      
      OK_other_obs_df <-  plyr::rbind.fill(as.data.frame(OK_other_obs_df), OK_other_obs)
      
    } else{
      #print(paste0("There is no observations in: ", stations_to_interp$date[j]))
      OK_other_obs           <- data.frame(data=NA)
      colnames(OK_other_obs) <- station
      OK_other_obs$date      <- stations_to_interp$date[j]
      
      OK_other_obs_df        <-  plyr::rbind.fill(as.data.frame(OK_other_obs_df), OK_other_obs)
      c <- c+1
    }
    
  }
  print(paste0("The number of days without any observational data is: ", c))
  
  # Select station data
  single_station_data <- data.frame(date            = OK_other_obs_df$date,
                                    obs             = AQMS_obs[,station],
                                    model           = df_all_model_raw_at_AQMS_gbm[,station], 
                                    day_of_year     = yday(OK_other_obs_df$date),
                                    day_of_week     = wday(OK_other_obs_df$date),
                                    other_obs       = OK_other_obs_df[,station])
  
  test_ind      <- which(is.na(single_station_data$obs))
  data_training <- single_station_data[-test_ind,]
  data_test     <- single_station_data[test_ind,]
  
  data_training   <- na.omit(data_training)
  data_test       <- data_test[!is.na(data_test$model),]
  data_test       <- data_test[!is.na(data_test$other_obs),]
  
  # Preliminary fit

  n.trees = 5000 #realment s'ha de provar i escollir l'òptim (si dona temps bucle amb 3-4 opcions per comparar) [la que dongui menor mse]
  Vx_formula <- "obs ~ model + day_of_year + day_of_week+ other_obs"
  gbm1 <- gbm(as.formula(Vx_formula),
              data = data_training,
              distribution = "gaussian",
              n.trees = n.trees,
              interaction.depth = 1, #amb 1 hauria de funcionar, no solem posar més de 10
              shrinkage = 0.01, #learning rate ni molt petit pk anirà lent ni molt gran pk te pasas [ns si bucle amb 3 proves]
              cv.folds = 5, #cross validation
              n.cores = NULL, # will use all cores by default
              verbose = FALSE)  
  gbm.fit <- gbm1
  
  # Data frame amb dos variables date(fecha + hora) i Tests_results/Obs_Test/Obs_Train (NA)
  Test_results_station  <-  data.frame(date=df_obs_years$date , Test_results = NA) 
  
  # Once you have decided on a final model you will likely want to use the model to predict on new observations. 
  res_test     <- predict( gbm1 , newdata=data_test[,c("model","day_of_year","day_of_week","other_obs")])
  
  
  Test_results <- data.frame(date = data_test[,'date'], Test_results=res_test )
  Test_results_station[Test_results_station$date%in%Test_results$date ,"Test_results"] <- Test_results[,"Test_results"]
  
  obs_ind      <- which(is.na(df_obs_years[,station]))
  
  df_obs_years[obs_ind,station] <- Test_results_station$Test_results[obs_ind]

  
}

saveRDS(object=df_obs_years, file=paste0(path_gbm,"gbm_gap_filling_results.rds"))

# 
# Summary_stats <- openair::modStats(data_single, obs = "Obs_Test_station", mod = "Test_results_station")
# print(Summary_stats)
# summary(data_single)




# 
# 
# openair::timePlot(data_single,
#                   #pollutant = c("Obs_Train_station","Obs_Test_station","Test_results_station","model"),
#                   pollutant = c("Obs_Test_station","Test_results_station"),
#                   group = TRUE,
#                   title=station,
#                   plot.type= "h",
#                   y.relation = "free",
#                   ylab="PM2,5 (ug/m3)",
#                   cols = c("blue", "red"),
#                   lty = c("solid","solid"),
#                   lwd = c(4,2), 
#                   name.pol = c("Obs Test station", "Test results station"))