# Measure the time it takes to execute the file
start_time_model <- Sys.time()

############################################################
#                                                          #
#                 Load Model                               #
#                                                          #
############################################################
#This script is called from UK to set model data:
contador <- 0
# We download the model by days
for (date in dates) {
  
  #date<-"20210101"
  conc_path_day  <- paste0(date, "00")
  file           <- paste0(name_pollutant, "_", conc_path_day, ".nc", sep="")
  file_path      <- paste0(model_path, file)
  
  if (file.exists(file_path)) {
    netcdf_model      <- RNetCDF::open.nc(file_path) #Dintre de la llibreria executem la funció open i aplica-ho al fitxer obs
    
    #Extraiem les dades del model
    model_day <- RNetCDF::var.get.nc(netcdf_model, name_pollutant) #Concentracions del NO2
    
    if(date==dates[1]){
      n_lat     <- dim(model_day)[1]
      n_lon     <- dim(model_day)[2]
      model     <- array(dim=c(n_lat,n_lon,0))
    }
    
    if(model_time_resolution=="hourly"){
      model_day      <- model_day[,,1:24]
    }else if(model_time_resolution=="daily"){
      model_day      <- model_day[,] #Meriem: revisar
    } else{
      print("Wrong time resolution")
    }
    
    model_day       <- model_day*mod_constant #les dades del model les tens en ppm i nosaltres ho volem en microg/m3
    
    if(model_time_resolution=="hourly" & time_resolution=="daily"){
      averaged_array  <- apply(model_day, c(1, 2), mean) # Matrix with the means of the day
      model_day       <- array(averaged_array, dim = c(nrow(averaged_array), ncol(averaged_array), 1)) # Arming the 3rd dimention (time)
    } else if( model_time_resolution=="daily" & time_resolution=="hourly"){
      print(paste0("WARNING: time resolution of the model and other data (observations mostly), do not match"))
    }
    
    }else{
      print(paste0("WARNING: There is no model or file on: ", date))
      countador   <- contador + 1
      empty_model <- NULL
      empty_model <- c(empty_model,date)
      
      if (!exists("n_lat") || !exists("n_lon")) {
        n_lat <- 278
        n_lon <- 298
        model <- array(dim=c(n_lat,n_lon,0))
      }
      
      if(time_resolution=="hourly"){
      model_day       <- array(NaN, dim=c(n_lat,n_lon,24)) #if we dont have data for a certain day its filled by NaN
      } else if (time_resolution=="daily"){
        model_day       <- array(NaN, dim=c(n_lat,n_lon,1)) #if we dont have data for a certain day its filled by NaN
      } else{
      print("Wrong model_time_resolution")
    }
    }
  model            <- abind(model,model_day,along=3)
  }

print(paste0("The number of missing raw model (CALIOPE) is: ", contador))
print(paste0("This days are: ", empty_model))

lat_model      <- RNetCDF::var.get.nc(netcdf_model, 'lat') #Latitut
lat_model      <- sapply(lat_model, as.numeric)            #era una large matrix, ara es una large numeric
lon_model      <- RNetCDF::var.get.nc(netcdf_model, 'lon') #Longitud
lon_model      <- sapply(lon_model, as.numeric)

coords_model   <- cbind(lon_model, lat_model)  
  
if (save_model){
  saveRDS(object=model, file=paste0(save_result,"model_",file_format, "_",pollutant,".rds"))
  saveRDS(object=coords_model, file=paste0(save_result,"coords_model_",file_format, "_",pollutant,".rds"))
  print("Model saved")
} else {
  print("Model have not been saved")
}

end_time_model    <- Sys.time()
temps_total_model <- end_time_model - start_time_model
print(paste0("local model, execution time :",temps_total_model, "s" ))
