# Measure the time it takes to execute the file
start_time_observations <- Sys.time()

############################################################
#                                                          #
#                 Load Observations                        #
#                                                          #
############################################################
#This script its called from UK_+_LOOCV to set the observations of Catalonia
#we have a doc where we have the reference of the stations that are in catalonia, if we want to change or amplify them we can change the coordinates of the box in set_stations script
print("Load observations")

if (file_format=="ghost"){
  
  if(time_resolution=="hourly"){
    print(paste0("Setting obs data as hourly mean data.", " ", "Using Ghost obs data"))
    # we prepare a matrix where we will save the observations:
    obs                  <- matrix(NA,ncol=(nrow(local_stations@data)),nrow=0)
    df_obs_all           <- data.frame(obs)
    dates_all            <- data.frame(matrix(NA,ncol= 1,nrow=0))
    
    #we transform from a matrix to a data frame, with the reference of the station as a name of the col
    colnames(df_obs_all) <- c(local_stations@data$ref_obs)
    colnames(dates_all)  <- c("date")
    # finally we extract the observations of certain year_month
    for (y_month in year_month) {
      #obs_path<-"/esarchive/scratch/mhajji/Meriem/RUN_UK_regional/LOOCV/observacions/obs_no2"
      netcdf_obs    <- RNetCDF::open.nc(paste0(obs_path, name_pollutant, "_", y_month, ".nc"))
      
      code_obs      <- RNetCDF::var.get.nc(netcdf_obs, 'station_reference')
      #station_name  <- RNetCDF::var.get.nc(netcdf_obs, "station_name")
      obs_all       <- RNetCDF::var.get.nc(netcdf_obs, name_pollutant)
      
      year            <- as.numeric(substr(y_month, 1, 4))
      month           <- as.numeric(substr(y_month, 5, 6))
      num_days        <- length(RNetCDF::var.get.nc(netcdf_obs, "time"))/24
      date_times_loc  <- data_frame_dates( c(year,month,01,00) , c(year,month,num_days,23) )
      dates_all       <- rbind(dates_all,date_times_loc)
      
      pos_local_stations  <- which(code_obs%in%local_stations@data$ref_obs)
      cat_obs             <- obs_all[,pos_local_stations]
      cat_ref             <- code_obs[pos_local_stations]
      #cat_names           <- station_name[pos_local_stations]
      
      obs_month               <- matrix(NA,ncol=length(cat_obs[1,]),nrow=length(cat_obs[,1])) 
      #cat_obs matriu de 72 col (estacions) i en aquest cas p.ex: 744 (31 dies de generx24h) i fem una obs month buida
      df_obs_month            <- data.frame(obs_month)
      colnames(df_obs_month)  <- paste0(cat_ref)
      df_obs_month[,]         <- cat_obs*obs_constant
      #df_obs_month            <- cbind(date = date_times_loc$date, df_obs_month)
      
      
      df_obs_all              <- rbind.fill(df_obs_all, df_obs_month) #in case there is no data on a certain date, it fills it with NA (rbind.fill)
      if ( ncol(df_obs_all)!=ncol(df_obs_month) ) {
        print(paste0("ERROR: number of monitoring stations is not constant when loading observations. ","Filled with NA in: ",y_month))
      }
    }
    
    df_obs_all <- cbind(dates_all,df_obs_all)
    
  }else{
    print("ERROR THE CODE IS NOT FINISHED YET")
    }

  }else if (file_format=="gene") {
    if(time_resolution=="hourly") {
    # If the data that we have is hourly (like no2 and o3):
    print(paste0("Setting obs data as hourly mean data.", " ", "Using Gene obs data"))
    
    #read the file and extract the station names, and the dates of the pollutant measures:
    netcdf_obs      <- read.csv(paste0(obs_path,"gene_",name_pollutant,"_",read_year,".csv"))
    #in netcdf_obs we had diferent variables, in numeric, logi or int type, just in case we pass them all to a numeric type
    
    ############################################################
    #           !!!!   WARNING!!!!                             #
    ############################################################
    # Hem posat tz= UTC, cal assegurar-nos que es així
    netcdf_obs$Date <- as.POSIXct(netcdf_obs$Date, format="%Y-%m-%d %H:%M:%S", tz = "UTC")
    netcdf_obs[,-1] <- dplyr::mutate_all(netcdf_obs[,-1],as.numeric)
    
    df_obs_all              <- netcdf_obs
    colnames(df_obs_all)[1] <- "date"
   
    } else if (time_resolution=="daily") {
    
    print(paste0("Setting obs data as daily mean data.", " ", "Using Gene obs data"))
    
    # If the data that we have is daily mean (like pm10 and pm2.5):
    
    #read the file and extract the station names, and the dates of the pollutant measures:
    netcdf_obs      <- read.csv(paste0(obs_path,"gene_",name_pollutant,"_",read_year,".csv"))
    netcdf_obs$Date <- as.Date(netcdf_obs$Date, format = "%Y-%m-%d")
    #in netcdf_obs we had diferent variables, in numeric, logi or int type, just in case we pass them all to a numeric type
    df_obs_all      <- dplyr::mutate_all(netcdf_obs[,-1],as.numeric)
    df_obs_all$Date <- netcdf_obs$Date 
    colnames(df_obs_all)[colnames(df_obs_all) == "Date"] <- "date"
    
  
    } else {
      print("wrong time_resolution")
    }
   
  } else {
    print("wrong file_format")
}


if (save_obs){
  saveRDS(object=df_obs_all, file=paste0(save_result,"df_obs_",file_format,"_",pollutant,".rds"))
  print("Observations saved")
} else {
  print("Observations have not been saved")
}

end_time_observations       <- Sys.time()
temps_total_observations    <- end_time_observations - start_time_observations
print(paste0("Load observations, execution time: ",temps_total_observations, "s" ))
