# Measure the time it takes to execute the file
start_time_UK_LOOCV <- Sys.time()
  

#LOOCV_total and model_raw_at_AQMS is created empty to fill it later with LOOCV and raw model data respectivelly

if (execute_uk) {
  UK_total_df           <- list(length=length_loop) #we will have a large list with a data.frame for each hour/day
}
if (execute_loocv) {
  LOOCV_total           <- matrix(NA,ncol=nrow(local_stations@data),nrow=0) #list(length=nrow(date_times)) #we will have a large list with a data.frame for each hour
  colnames(LOOCV_total) <- local_stations$ref_obs #we want a matrix with empty data and the name of stations
}

if (execute_uk=="FALSE" && execute_loocv=="FALSE") {
  print("The user did not choose any method to apply")
}

model_raw_at_AQMS <- model_raw_at_AQMS_df

# Lists to save the slope and the interception between observations and model:

slope_v  <-  array(NA,dim = length_loop)
inter_v  <-  array(NA,dim = length_loop)
r_v      <-  array(NA,dim = length_loop)


############################################################
#                                                          #
#                 Loop UK - LOOCV                          #
#                                                          #
############################################################
print("Loop UK - LOOCV")
#starting OK and LOOCV:
for (hour in 1:length_loop) {
  print(paste0("loop UK or LOOCV number:",hour))
  
  #extract observations hourly
  df_obs_hour <- subset(df_obs_sub[hour,], select = -c(date))
 
  # #To avoid what we suppose wrong observations, we will subtitute the values higher than limit microg/m3
  # w <- length(which(df_obs_hour > limit, arr.ind = TRUE)) # Counts the nª of values that exceed the limit
  # if (w >= 1) {
  #   df_obs[df_obs > limit] <- limit # Substitute the values higher than the limit, for the limit value
  #   print(paste0("The observations has been modified to exceed ", limit, " límit value. The number of observations modified are : ", w, " in the hour:", hour))
  # }
  
  # Get the row and column indices of values that exceed the limit
  indices <- which(df_obs_hour > limit, arr.ind = TRUE)
  
  # Check if any values exceed the limit
  if (nrow(indices) > 0) {
    # Modify the values that exceed the limit
    df_obs_hour[indices] <- limit
    
    # Print a message
    message(paste0("The observations have been modified to not exceed ", limit, " microg/m3. The number of observations modified are: ", nrow(indices), " in day/hour: ", hour))
  }
  
  #raw model:
  df_model_uk                  <- model[,,hour]
  
  if (!any(is.na(df_model_uk))){
    df_model_uk                  <- data.frame(lat=lat_model,lon=lon_model,pollutant_model=sapply(df_model_uk,as.matrix))
    coordinates(df_model_uk)     <- ~lon+lat
    sp::proj4string(df_model_uk) <- crs_latlon
    
    #config the coordinates in UTM
    if (compute_in_UTM) {
      df_model_uk <- spTransform(df_model_uk, sp::CRS(crs_utm))
    }
    
    
    ok_hour <- model_raw_at_AQMS[hour,-which(names(model_raw_at_AQMS)=="date")]
  
    df_loocv                     <- data.frame(lat=local_stations@coords[,2],
                                               lon=local_stations@coords[,1],
                                               pollutant_model=as.numeric(ok_hour[1,]),
                                               ref_obs=local_stations@data$ref_obs) 
    
    df_obs_loocv <- data.frame(pollutant_obs = as.numeric(df_obs_hour[1,]) , ref_obs = colnames(df_obs_hour))
    
    llista_loocv <- merge(df_loocv, df_obs_loocv, by = "ref_obs")
    #EN EL FUTUR POSAR QUE VAGI GUARDANT RESULTATS DE MES EN MES, SINO ES UN COÑAZO PERDRE-HO TOT
    if (!all(is.na(llista_loocv$pollutant_obs))){
      
      llista_loocv <- na.omit(llista_loocv) 
      
      coordinates(llista_loocv)        <- ~lon+lat
      sp::proj4string(llista_loocv)    <- crs_latlon
      
      #config the coordinates in UTM
      if (compute_in_UTM) {
        llista_loocv <- spTransform(llista_loocv, sp::CRS(crs_utm))
      } else {
        sp::proj4string(llista_loocv) <- crs_latlon
      }
      
      # Compute LOOCV variogram
      
      pollutant.fit_loocv <- automap::autofitVariogram(formula=pollutant_obs~pollutant_model,input_data=llista_loocv, verbose=FALSE)
      
      #Emergency Variogram if range of the current Variogram it's negative
      if(any(pollutant.fit_loocv$var_model$range<0)){
        print('UK Variogram had negative range')
        pollutant.fit_loocv=readRDS(variogram_uk)
      }
      
      #pollutant.fit_loocv$var_model[2,"range"] <- pollutant.fit_loocv$var_model[2,"range"]
      
      #coef(lm(pollutant_obs ~  pollutant_model , llista_uk))['pollutant_model'] is the same as slope:
      slope <- summary(lm(pollutant_obs ~  pollutant_model , llista_loocv))$coefficients[,1][-1]
      interc <- ((lm(pollutant_obs ~  pollutant_model , llista_loocv))$coefficients)[1]
      r <- cor(llista_loocv$pollutant_obs,llista_loocv$pollutant_model, method = "pearson")
      #p     <-summary(lm(pollutant_obs ~  pollutant_model , llista_uk))$coefficients[,4][-1] #p value.
      
      #plot(llista_loocv$pollutant_model,llista_loocv$pollutant_obs)
      
      slope_v[hour]  <-  slope
      inter_v[hour]  <- interc
      r_v[hour]      <- r
      
      
      if(slope[1] < -20){
        
        if(execute_uk){
          # UK
          uk_interpolation              <- as.data.frame(df_model_uk$pollutant_model)
          colnames(uk_interpolation)    <- c("var1.pred")
          coordinates(uk_interpolation) <- df_model_uk@coords
          
          #config the coordinates in UTM
          if (compute_in_UTM) {
            sp::proj4string(uk_interpolation) <- crs_utm
          } else {
            sp::proj4string(uk_interpolation) <- crs_latlon
          }
        }
        
        
        if(execute_loocv){
          # LOOCV
          #loocv_interpolation              <- as.data.frame(llista_loocv$pollutant_model)
          #colnames(loocv_interpolation)    <- c("var1.pred")
          loocv_interpolation              <- data.frame(var1.pred= array(NA,dim=length(llista_loocv$pollutant_model) ) )
          coordinates(loocv_interpolation) <- llista_loocv@coords
          
          #config the coordinates in UTM
          if (compute_in_UTM) {
            sp::proj4string(loocv_interpolation) <- crs_utm
          } else {
            sp::proj4string(loocv_interpolation) <- crs_latlon
          }
        }
        
        print(paste0("The slope in hour ", hour, " is ", slope, ". We did not apply LOOCV, raw-model value is picked for this hour"))
        
      }else {
        
        #Save the first variogram executed for a certain pollutant if we already don't have one saved:
        if (!file.exists(variogram_uk)) {
          saveRDS(pollutant.fit_loocv,variogram_uk)
        }
        
        if(execute_uk){
          # UK
          uk_gstat          <- gstat(formula=pollutant_obs~pollutant_model,data=llista_loocv, model = pollutant.fit_loocv$var_model)
          uk_interpolation  <- predict(object=uk_gstat, newdata=df_model_uk)
        }
        
        if(execute_loocv){
          # LOOCV
          loocv_interpolation  <- krige.cv(pollutant_obs~pollutant_model,llista_loocv,pollutant.fit_loocv$var_model)
        }
      }
      
      if(execute_uk){
        UK_total_df[[hour]]       <- uk_interpolation@data$var1.pred
      }
      if(execute_loocv){
        uk_loocv_h             <- matrix(data=loocv_interpolation@data$var1.pred, nrow=1)
        colnames(uk_loocv_h)   <- llista_loocv@data$ref_obs
        LOOCV_total            <- plyr::rbind.fill(as.data.frame(LOOCV_total),as.data.frame(uk_loocv_h))
      }
      
    } else{
      print(paste0("WARNING: The observations are corrupted or non-existen in hour or day : ", hour))
      if(execute_uk){
        UK_total_df[[hour]]       <- rep(NA, length(df_model_uk))
      }
      if(execute_loocv){ #M: no m'acaba de convèncer
        
        uk_loocv_h             <- matrix(NA, nrow = 1, ncol = nrow(local_stations))
        colnames(uk_loocv_h )  <- local_stations@data$ref_obs
        LOOCV_total            <- plyr::rbind.fill(as.data.frame(LOOCV_total),as.data.frame(uk_loocv_h))
      }
    }
    
  } else{ #Arreglar
    print(paste0("WARNING: The model is corrupted or non-existen in hour or day : ", hour))
    if(execute_uk){
      UK_total_df[[hour]]       <- rep(NA, length(df_model_uk))
    }
    if(execute_loocv){ #M: no m'acaba de convèncer
    
      uk_loocv_h             <- matrix(NA, nrow = 1, ncol = nrow(local_stations))
      colnames(uk_loocv_h )  <- local_stations@data$ref_obs
      LOOCV_total            <- plyr::rbind.fill(as.data.frame(LOOCV_total),as.data.frame(uk_loocv_h))
    }
    
  }
  
}


# Set LOOCV_total_df with dates
if(execute_loocv){
  LOOCV_total_df        <- as.data.frame(LOOCV_total)
  for(i in 1:nrow(local_stations)){
    LOOCV_total_df[[i]] <- replace(LOOCV_total_df[[i]], LOOCV_total_df[[i]]<0, 0)
  } #Fer lo mateix per UK, + imprimir quants valors negatius ha posat també !!!!!!!!!!!!
  
  
  if(time_resolution=="hourly"){
    #UK_LOOCV_total_df <- cbind(dates,UK_LOOCV_total_df_wd)
    LOOCV_total_df        <- cbind(date = date_times$date, LOOCV_total_df)
    #Verification of negative values 'which(model_raw_at_AQMS_df<0)'. 
    #At the moment we don't apply it to model_raw_at_AQMS because it doesn't have negative values
  }else if(time_resolution=="daily"){
    LOOCV_total_df        <- cbind(date = dates, LOOCV_total_df)
    LOOCV_total_df$date <- as.Date(LOOCV_total_df$date, format = "%Y%m%d")
    
  }else{
    print("Wrong time_resolution")
  }
}
  


if (save_LOOCV){
  if(execute_uk){
    saveRDS(object=UK_total_df, file = paste0(save_result,"UK_",file_format, "_",pollutant,"_df.rds")) 
    print("UK saved")
  }
  if(execute_loocv){
    saveRDS(object=LOOCV_total_df, file = paste0(save_result,"LOOCV_",file_format, "_",pollutant,"_df.rds"))
    print("LOOCV saved")
  }
  
} else {
  print("LOOCV - UK have not been saved")
}

end_time_UK_LOOCV     <- Sys.time()
temps_total_UK_LOOCV  <- end_time_UK_LOOCV  - start_time_UK_LOOCV 
print(paste0("UK or LOOCV, execution time:",temps_total_UK_LOOCV))
