############################################################
#                                                          #
#              evaluation statistics                       #
#                                                          #
############################################################

# it alerts us if the dates in model do not coincide with the dates of the obs

# if ( any(date_times$date != df_obs_sub$date))  {
#   print("Warning: dates from model and observations are different")
# }
# no se si treure'l o bé, posar un if segons la time_resolution

#-------------------------------------------------------------------------------
# We will generate a list with the names in local stations that are in df_obs_sub:
names_local_stations <- as.list(intersect(local_stations@data$ref_obs, colnames(df_obs_sub)))

hourly_pollutants <- c("no2", "o3")
daily_pollutants  <- c("pm10", "pm2p5")

#We need a data.frame with raw_model and LOOCV_model to apply the stadistics

# Comparation using all stations concatenated:
raw_model_total         <- NULL
UK_model_total          <- NULL 
geostats_results        <- NULL

for (station_code in names_local_stations) {
  # station_code <- names_local_stations[[1]]
  obs_station       <- df_obs_sub[ ,station_code]
  raw_model_estacio <- data.frame(obs = obs_station,
                                  mod = model_raw_at_AQMS_df[ ,station_code], 
                                  model="raw_model")
  UK_model_estacio <-   data.frame(obs = obs_station,
                                   mod = LOOCV_total_df[ ,station_code],
                                   model="UK_model")

  # Statistics for a single station
  Raw_geostats    <- openair::modStats(raw_model_estacio, obs = "obs", mod = "mod", type = "model")
  UK_geostats     <- openair::modStats(UK_model_estacio, obs = "obs", mod = "mod", type = "model")

  geostats_station <- data.frame(x=local_stations@coords[which(local_stations@data$ref_obs == station_code),1],
                                 y=local_stations@coords[which(local_stations@data$ref_obs == station_code),2],
                                 COE_raw = Raw_geostats$COE,
                                 COE_UK = UK_geostats$COE,
                                 MB_raw = Raw_geostats$MB,
                                 MB_UK = UK_geostats$MB)
  
  # Gather all stations
  geostats_results    <- rbind(geostats_results,geostats_station)
  #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
  raw_model_total     <- rbind(raw_model_total,raw_model_estacio) 
  UK_model_total      <- rbind(UK_model_total,UK_model_estacio)
}

mod_obs_data <- rbind(raw_model_total,
                      UK_model_total)

#Openair function that gives us the stadistics we want

Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)
png(file = paste0(save_general_statistics,"All_stations_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()


# 
# # ##
# # ## Map Statistics
# # ##
# geostats_results <- na.omit(geostats_results)
# coordinates(geostats_results)        <- ~ x + y
# sp::proj4string(geostats_results)    <- crs_latlon
# 
# coe_uk <- mapview::mapview(geostats_results, zcol = "COE_UK", legend = TRUE,at=c(-30,-5,-1,-0.5,0,0.5,1))
# mb_uk  <- mapview::mapview(geostats_results, zcol = "MB_UK", legend = TRUE,at=c(-15,-10,-5,0,5,10,15))

#to save it in htm:
# mapshot(coe_uk,paste0(save_result,"COE_UK.html"))
# mapshot(mb_uk,paste0(save_result,"MB_UK.html"))

print("Statistic for all stations concateneited: ")

#-------------------------------------------------------------------------------
# PROVA PER DETECTAR L'ESTACIÓ ON L'ESTADÍSTIC DONGUI MALAMENT
counter <- 0
print(paste0("starting evaluation for each local station"))

for (station_code in names_local_stations) {
   # station_code <- names_local_stations[[1]]
# mapshot(coe_uk,paste0(save_result,"COE_UK.html"))
# mapshot(mb_uk,paste0(save_result,"MB_UK.html"))
   geostats_results        <- NULL
   
   obs_station       <- df_obs_sub[ ,station_code]
   raw_model_estacio <- data.frame(obs = obs_station,
                                   mod = model_raw_at_AQMS_df[ ,station_code], 
                                   model="raw_model")
   UK_model_estacio <-   data.frame(obs = obs_station,
                                    mod = LOOCV_total_df[ ,station_code],
                                    model="UK_model")
   
   # Statistics for a single station
   Raw_geostats    <- openair::modStats(raw_model_estacio, obs = "obs", mod = "mod", type = "model")
   UK_geostats     <- openair::modStats(UK_model_estacio, obs = "obs", mod = "mod", type = "model")
   
   
   #raw_model_total and corr_model_total are a data frame with three lists (obs, mod, model(raw or corrected))
   raw_model_total     <- rbind(raw_model_total,raw_model_estacio) 
   UK_model_total      <- rbind(UK_model_total,UK_model_estacio)
   
   mod_obs_data <- rbind(raw_model_total,
                         UK_model_total)
   
   #Openair function that gives us the stadistics we want
   counter<-counter+1
   print(counter)
   print(station_code)
   Summary_stats <- openair::modStats(mod_obs_data, obs = "obs", mod = "mod", type = "model")
   print(Summary_stats)
}



#-------------------------------------------------------------------------------
# Mean value for each hour all stations:
obs_total_mean            <- matrix(nrow = length(df_obs_sub$date), ncol = 0) 
raw_model_total_mean      <- matrix(nrow = length(df_obs_sub$date), ncol = 0)
UK_model_total_mean       <- matrix(nrow = length(df_obs_sub$date), ncol = 0)

for (station_code in names_local_stations) {
  obs_station <- df_obs_sub[ ,station_code]
  # station_code <- local_stations@data$code[1]
  
  raw_model_estacio  <-  model_raw_at_AQMS_df[ ,station_code]
  UK_model_estacio   <-  LOOCV_total_df[ ,station_code]
  
  obs_total_mean          <- cbind(obs_total_mean,obs_station)
  raw_model_total_mean    <- cbind(raw_model_total_mean,raw_model_estacio)
  UK_model_total_mean     <- cbind(UK_model_total_mean,UK_model_estacio)
}

raw_model_mean      <- data.frame( obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(raw_model_total_mean,na.rm=TRUE),
                                   model="raw_model")

UK_model_mean     <- data.frame(   obs = rowMeans(obs_total_mean,na.rm=TRUE),
                                   mod = rowMeans(UK_model_total_mean,na.rm=TRUE),
                                   model="UK_model")

mod_obs_data_mean   <- rbind(raw_model_mean,
                             UK_model_mean)


Summary_stats <- openair::modStats(mod_obs_data_mean, obs = "obs", mod = "mod", type = "model")
print(Summary_stats)

png(file = paste0(save_general_statistics,"All_stations_hourly_",pollutant,"_stats.png"), width = 1472, height = 540)
gridExtra::grid.table(Summary_stats)
dev.off()

print("Statistic considering the mean hour for all stations: ")


#-------------------------------------------------------------------------------
# Legislative values (daily max for no2):

#NOTA: Gene LOOCV s'avalua a escala horària, mitjana diària i màxim diàri
if( pollutant=="nox"){
   
   
   ############################################################
   #                          MÀXIM                           #
   ############################################################
   # Group data by date and find maximum value for each local station
   #aggregate de la llibreria sp
   max_obs_daily               <- aggregate(subset(df_obs_sub, select=names(df_obs_sub) %in% names_local_stations),
                                            by=list(date=format(df_obs_sub$date, "%Y-%m-%d")), max)
   
   max_raw_mod_daily           <- aggregate(subset(model_raw_at_AQMS_df, select=names(model_raw_at_AQMS_df) %in% names_local_stations),
                                            by=list(date=format(model_raw_at_AQMS_df$date, "%Y-%m-%d")), max)
   
   max_corrected_loocv_daily   <- aggregate(subset(LOOCV_total_df, select=names(LOOCV_total_df) %in% names_local_stations),
                                            by=list(date=format(LOOCV_total_df$date, "%Y-%m-%d")), max)
   
   # max_burnes_daily            <- aggregate(subset(df_burnes_all, select=names(df_burnes_all) %in% names_local_stations),
   #                                          by=list(date=format(df_burnes_all$date, "%Y-%m-%d")), max)
   ############################################################
   #                             19 MAX                       #
   ############################################################
   #Primer agafar els màxims diaris, i d'aqui els 19 è
   max19_obs <- NULL
   
   obs_matrix           <- matrix(NA,ncol=length(names_local_stations),nrow=length_loop)
   obs_df               <- data.frame(obs_matrix)
   colnames(obs_df)     <- names_local_stations
   
   for(station_code in names_local_stations){
      #station_code <- names_local_stations[[1]]
      obs_df[,station_code] <- sort(df_obs_sub[,station_code], decreasing=TRUE,na.last=TRUE)
   }
   
   df_19max_obs <- obs_df[1,]
   
   
   #anual
   # max19_obs_annual <- aggregate(subset(df_obs_sub, select = names(df_obs_sub) %in% names_local_stations),
   #                               by = list(station = names_local_stations,
   #                                         year = format(df_obs_sub$date, "%Y")),
   #                               FUN = function(x) {
   #                                 sort(x, decreasing = TRUE)[19]
   #                               })
   
   
   ############################################################
   #                          MITJANA                         #
   ############################################################
   # Group data by date and find mean value for each local station
   mean_obs_daily               <- aggregate(subset(df_obs_sub, select=names(df_obs_sub) %in% names_local_stations),
                                             by=list(date=format(df_obs_sub$date, "%Y-%m-%d")), mean)
   
   # Per que sigui media anual:
   #mean_obs_yearly <- aggregate(subset(df_obs_sub, select=names(df_obs_sub) %in% names_local_stations),
   #by=list(year=format(df_obs_sub$date, "%Y")), mean)
   
   
   mean_raw_mod_daily           <- aggregate(subset(model_raw_at_AQMS_df, select=names(model_raw_at_AQMS_df) %in% names_local_stations),
                                             by=list(date=format(model_raw_at_AQMS_df$date, "%Y-%m-%d")), max)
   
   mean_corrected_loocv_daily   <- aggregate(subset(LOOCV_total_df, select=names(LOOCV_total_df) %in% names_local_stations),
                                             by=list(date=format(LOOCV_total_df$date, "%Y-%m-%d")), mean)
   
   # mean_burnes_daily            <- aggregate(subset(df_burnes_all, select=names(df_burnes_all) %in% names_local_stations),
   #                                          by=list(date=format(df_burnes_all$date, "%Y-%m-%d")), mean)
   
   ############################################################
   #                          MAX ALL AQMS                    #
   ############################################################
   # Group data by date and find maximum value for each local station
   #aggregate de la llibreria sp
   max_obs_daily               <- df_obs_sub
   
   max_raw_mod_daily           <- model_raw_at_AQMS_df
   
   max_corrected_loocv_daily   <- aggregate(subset(LOOCV_total_df, select=names(LOOCV_total_df) %in% names_local_stations),
                                            by=list(date=format(LOOCV_total_df$date, "%Y-%m-%d")), max)
   
   # max_burnes_daily            <- aggregate(subset(df_burnes_all, select=names(df_burnes_all) %in% names_local_stations),
   #                                          by=list(date=format(df_burnes_all$date, "%Y-%m-%d")), max)
   
   
}


#-------------------------------------------------------------------------------

#-------------------------------------------------------------------------------
# Per una estació:
# First, we will evalue the raw model. To do so, we need a data frame with obs and the raw model. Obv we choose df_obs_sub as obs
# colnames(model_raw_at_AQMS)[1] --> We do the stadistics for each station
# 
# raw_model <- data.frame( obs = df_obs_sub[ ,colnames(model_raw_at_AQMS)[1] ],
#                          mod = model_raw_at_AQMS_df[ ,colnames(model_raw_at_AQMS)[1]],
#                          model="raw_model")
# 
# #Aqui posarem el LOOC_total_df  que s'ha anat generant en el bucle amb el model corretgit
# corr_model <- data.frame( obs = df_obs_sub[ ,colnames(model_raw_at_AQMS)[1] ],
#                           mod = LOOCV_total_df[ ,colnames(model_raw_at_AQMS)[1]],
#                           model="corr_model")
# 
# #tres columnes: obs, mod i quin tipus de model es 'model': que pot ser raw_model o corr_model. Primer et posa el raw i seguidament el corr
# mod_obs_data_1 <- rbind(raw_model,
#                       corr_model)
# 
# # create evaluation statistics
# print("Example of the statistics for one station: ")
# Summary_stats <- openair::modStats(mod_obs_data_1, obs = "obs", mod = "mod", type = "model")
# print(Summary_stats)


